import React, { useEffect, useMemo, useState } from "react";
import type { BannerType, IBusiness } from "../typings/business";
import type { BannerDataType } from "../data/banner";
import { BANNER_DATA } from "../data/banner";
import { SEGMENT_EVENTS } from "../data/segmentEvents";
import { TouchableOpacity, View, ViewStyle } from "react-native";
import { showMessage } from "react-native-flash-message";
import { COLOR } from "../data/color";
import { useLMS } from "../context/context";
import { Button, Text } from "react-native-paper";
import analytics from "../utils/segment";
import Feather from "react-native-vector-icons/Feather";
import Ionicons from "react-native-vector-icons/Ionicons";

type OFHeaderProps = {
  onPress?: (args: { type: BannerType; contractUrl: string }) => void;
  onClose?(): void;
};

export const Banner: React.ComponentType<OFHeaderProps> = ({
  onClose,
  onPress,
}) => {
  const [detail, setDetail] = useState<IBusiness | undefined>();
  const { info, userDetail } = useLMS();

  const status = useMemo(() => {
    let contractUrl, contractValid, reason;
    if (detail) {
      contractUrl = detail.kyb?.facilities.filter(
        (d: any) => d.status === "contract_pending",
      )[0]?.envelope_url;
      contractValid = detail.kyb?.facilities.filter(
        (d: any) => d.status === "contract_valid",
      );
      reason = detail.kyb?.statuses?.filter(
        (d: any) => d.status === "rejected",
      );
    }
    return {
      contractUrl,
      contractValid,
      reason: reason && reason.length > 0 ? reason[0].reason : "",
    };
  }, [detail]);

  const RenderBanner: React.ComponentType<{
    data: BannerDataType;
    color: string;
    disabled?: boolean;
    event?: string;
    iconColor?: string;
  }> = ({ data, color, disabled, event, iconColor }) => {
    return (
      <View
        style={{
          ...(styles.financeContainer as ViewStyle),
          backgroundColor: color,
        }}
      >
        <View
          style={{
            ...styles.flexRow,
            ...styles.iconWrapper,
          }}
        >
          <Feather name={"clock"} size={28} color={iconColor} />
          <TouchableOpacity
            onPress={() => {
              onClose?.();
            }}
          >
            <Ionicons name={"close"} size={20} color={"#000"} />
          </TouchableOpacity>
        </View>

        <View style={styles.gap} />

        <Text variant={"bodyLarge"} style={{ fontWeight: 500 } as ViewStyle}>
          {data.title}
        </Text>
        <Text variant={"bodySmall"}>{data.description}</Text>

        <View style={styles.gap} />

        {data.actionTitle && (
          <Button
            mode={"contained"}
            disabled={disabled}
            style={styles.button}
            onPress={() => {
              if (event) {
                analytics.track(event);
              }
              onPress?.({
                type: data.type,
                contractUrl: status.contractUrl || "",
              });
            }}
          >
            {data.actionTitle}
          </Button>
        )}
      </View>
    );
  };

  useEffect(() => {
    if (!info) {
      showMessage({ message: "Email is required", type: "danger" });
    } else {
      setDetail(userDetail);
    }
  }, [info, userDetail]);

  return (
    <View style={{ ...(styles.container as ViewStyle), flexWrap: "nowrap" }}>
      {detail && (
        <>
          {!detail.kyb && (
            <RenderBanner
              data={BANNER_DATA().REGISTER as BannerDataType}
              color={COLOR.info}
              iconColor={"#75B1FF"}
            />
          )}
          {detail?.kyb && detail.kyb?.status === "pending" && (
            <RenderBanner
              data={BANNER_DATA().PENDING as BannerDataType}
              color={COLOR.warning}
              iconColor={"#D9A514"}
            />
          )}
          {detail.kyb &&
            detail.kyb.status === "approved" &&
            status.contractValid?.length === 0 && (
              <RenderBanner
                data={
                  BANNER_DATA(
                    undefined,
                    !status.contractUrl
                      ? "You will receive contract soon"
                      : "Review and sign contract",
                  ).APPROVED as BannerDataType
                }
                color={COLOR.info}
                disabled={!status.contractUrl}
                event={SEGMENT_EVENTS.CLICK_SIGN_CONTRACT}
                iconColor={"#75B1FF"}
              />
            )}
          {detail.kyb && detail.kyb.status === "rejected" && (
            <RenderBanner
              data={BANNER_DATA(status?.reason).REJECTED as BannerDataType}
              color={COLOR.error}
              event={SEGMENT_EVENTS.CLICK_KYC_REGISTRATION}
              iconColor={"#FC9086"}
            />
          )}
        </>
      )}
    </View>
  );
};

const styles = {
  container: {
    flexDirection: "column",
    border: "1px solid red",
    flex: 1,
    width: "100%",
  },
  financeContainer: {
    display: "flex",
    flexDirection: "column",
    padding: 12,
    flexWrap: "nowrap",
    alignItems: "flex-start",
    justifyContent: "space-between",
  },
  button: { width: "100%" } as ViewStyle,
  gap: {
    width: 10,
    height: 10,
  },
  wrapper: {
    flexDirection: "column",
  },
  iconWrapper: {
    width: "100%",
    justifyContent: "space-between",
  } as ViewStyle,
  flexRow: {
    display: "flex",
    flexDirection: "row",
  } as ViewStyle,
  flexCol: {
    display: "flex",
    flexDirection: "column",
  } as ViewStyle,
};
