import { StyleSheet, View } from "react-native";
import * as Yup from "yup";
import { LoanPost } from "../typings/loan";
import React, { useState } from "react";
import { CREATE_LOAN, PREDICTION } from "../data";
import { Button, Divider, Text, useTheme } from "react-native-paper";
import { useLMS } from "../context/context";
import { useAppErrors } from "../hooks";
import { arrayLength, removeArrays } from "../helpers";
import analytics from "../utils/segment";
import { SEGMENT_EVENTS } from "../data/segmentEvents";
import { PLATFORM } from "../data/constant";
import queryString from "query-string";
import { showMessage } from "react-native-flash-message";
import { FieldTextInput, Gutter, UploadAttachment } from "../components";
import { Formik } from "formik";

const validationSchema = Yup.object().shape({
  required_financing: Yup.number()
    .required("Amount is required")
    .min(1, "Amount must be greater than zero"),
});

type FinanceApplicationFormProps = {
  loanId?: string;
  data?: LoanPost;
  onSuccess?(loan: any): void;
};

export const FinanceApplicationForm: React.ComponentType<
  FinanceApplicationFormProps
> = ({ loanId, data, onSuccess }) => {
  const [prediction, setPrediction] = useState<PREDICTION>();
  const theme = useTheme();
  const { axiosInstance, info, userDetail } = useLMS();
  const { setAppError } = useAppErrors();

  const calculateAmount = async (tenure: string, principal: string) => {
    try {
      const response = await axiosInstance.post(
        "/v1/external-lending/sme/loans/predict",

        {
          user_id: userDetail?.kyb.id,
          tenure,
          principal_amount: principal,
        },
      );
      setPrediction(response.data.balances);
    } catch (e: any) {
      setAppError(e.response.data);
    }
  };

  const onSuccessResponse = (response: any, action: any, value: any) => {
    const documentLength = arrayLength(value);
    onSuccess?.(response);
    action.resetForm({
      values: CREATE_LOAN,
    });
    setPrediction(undefined);
    showMessage({
      message: "Successfully Submitted",
      type: "success",
    });

    analytics.track(SEGMENT_EVENTS.COMPLETE_FINANCE_APPLICATION_FORM, {
      ...removeArrays(value),
      ...PLATFORM,
      attached_documents_count: documentLength,
    });
  };

  const onSubmit = async (value: any, action: any) => {
    try {
      action.setSubmitting(true);

      if (!data) {
        const { data: response } = await axiosInstance.post(
          `/v1/external-lending/sme/loans?${queryString.stringify(info || {})}`,
          {
            ...value,
          },
        );
        onSuccessResponse(response, action, value);
      } else {
        const { data: response } = await axiosInstance.patch(
          `/v1/external-lending/sme/loans/${loanId}`,
          {
            ...value,
            ...info,
          },
        );
        onSuccessResponse(response, action, value);
      }
    } catch (e: any) {
      setAppError(e?.response?.data);
    } finally {
      action.setSubmitting(false);
    }
  };

  return (
    <View style={styles.container}>
      <Formik
        initialValues={data || CREATE_LOAN}
        validationSchema={validationSchema}
        onSubmit={onSubmit}
      >
        {({ values, handleSubmit, isSubmitting, isValid, dirty }) => (
          <>
            <View>
              <Text variant={"headlineSmall"} style={{ fontWeight: "500" }}>
                Apply for Financing
              </Text>
              <Gutter spacing={0.1} />
              <Text variant={"bodyLarge"}>
                Please provide us details of your inventory/purchase
                order/invoice.
              </Text>
              <Gutter spacing={1.5} />
              <UploadAttachment
                buttonLabel={"ATTACH DOCUMENT"}
                name={"invoice_data"}
                title={"Documents (invoice/purchase order/inventory)"}
              />
              <Gutter spacing={0.3} />
              <FieldTextInput
                title={"Financing Amount (PKR)"}
                name="required_financing"
                placeholder={"0"}
                onBlur={(e) =>
                  calculateAmount((values?.tenure as string) || "0", e)
                }
              />
              <Gutter spacing={0.3} />
              <FieldTextInput
                title={"Financing Term (days)"}
                name="tenure"
                placeholder={"0"}
                onChange={(e) =>
                  calculateAmount(e, values.required_financing as string)
                }
              />
              <Gutter />
              <Text variant={"titleSmall"} style={styles.label}>
                Principal Amount
              </Text>
              <Gutter spacing={0.1} />
              <Text variant={"bodyLarge"}>
                PKR{" "}
                {(
                  parseFloat(`${values.required_financing}`) || 0
                ).toLocaleString()}
              </Text>
              <Gutter />
              <Text variant={"titleSmall"} style={styles.label}>
                Total Markup
              </Text>
              <Text variant={"bodyLarge"}>PKR {prediction?.markupPending}</Text>
              <Gutter spacing={0.7} />
              <Divider />
              <Gutter spacing={0.7} />
              <Text variant={"titleSmall"}>Total Net Payment Due</Text>
              <Text variant={"titleSmall"} style={styles.label}>
                PKR{" "}
                {(
                  (parseFloat(`${prediction?.amountPending}`) || 0) +
                  (prediction?.markupPending || 0)
                ).toLocaleString()}
              </Text>
              <Gutter spacing={2} />
              <Button
                style={styles.button}
                mode="contained"
                onPress={() => handleSubmit()}
                disabled={isSubmitting || !isValid || !dirty}
              >
                Submit
              </Button>
            </View>
          </>
        )}
      </Formik>
    </View>
  );
};

const styles = StyleSheet.create({
  container: {
    minHeight: 3000,
    display: "flex",
    flex: 1,
    padding: 16,
  },
  label: { color: "rgb(124, 117, 126)" },
  scrollViewContent: {
    padding: 16,
  },
  attachment: {
    marginBottom: 10,
  },
  button: { borderRadius: 4 },
});

/*
  <Typography
    variant={'caption'}
    color={theme.palette.grey['700']}
  >
    Total Markup
  </Typography>
  <Typography variant={'body2'}>
    PKR {prediction?.markupPending}
  </Typography>
  <Gutter spacing={0.4} />
  <HorizontalBar />
  <Gutter spacing={0.4} />
  <Typography
    variant={'caption'}
    color={theme.palette.grey['700']}
  >
    Total Net Payment Due
  </Typography>
  <Typography variant={'body2'} fontWeight={600}>
    PKR{' '}
    {(
      (parseFloat(
        `${prediction?.amountPending}`
      ) || 0) +
      (prediction?.markupPending || 0)
    ).toLocaleString()}
  </Typography>
</Box>
<StyledRow>
  <StyledPrimaryButton
    variant={'outlined'}
    onClick={() => {
      //
    }}
    disabled={isSubmitting}
  >
    <PrimaryButtonWrapper>
      Cancel
    </PrimaryButtonWrapper>
  </StyledPrimaryButton>
  <Gutter gap={1} />
  <StyledPrimaryButton
    variant={'contained'}
    onClick={() => handleSubmit()}
    disabled={isSubmitting || !isValid}
  >
    <PrimaryButtonWrapper>
      Submit
    </PrimaryButtonWrapper>
  </StyledPrimaryButton>
</StyledRow>*/
