import React, {useCallback, useEffect, useState} from 'react';
import queryString from 'query-string';
import {FilteredLoanStatus, ILoan, LoanListType} from '../typings/loan';
import {useLMS} from '../context/context';
import {useAppErrors} from '../hooks';
import {APPLICATION_STATUS, FILTER_BY_STATUS} from '../data/Loanlist';
import {
  ScrollView,
  StyleSheet,
  TouchableOpacity,
  useWindowDimensions,
  View,
} from 'react-native';
import {Text, TextInput, useTheme} from 'react-native-paper';
import RNPickerSelect from 'react-native-picker-select';
import {SEGMENT_EVENTS} from '../data/segmentEvents';
import analytics from '../utils/segment';
import {LoansTable} from '../screen/components/loanItem';
import {SceneMap, TabBar, TabView} from 'react-native-tab-view';
import {ApplicationTable} from '../screen/components/ApplicationList';

type LoanListProps = {
  onPress?(type: LoanListType, loan: ILoan): void;
};

export const LoanList: React.ComponentType<LoanListProps> = ({onPress}) => {
  const [search, setSearch] = useState('');
  const [filter, setFilter] = useState<FilteredLoanStatus>(
    FilteredLoanStatus.ALL_LOANS,
  );
  const [filteredData, setFilteredData] = useState<ILoan[]>([]);
  const {axiosInstance, info} = useLMS();
  const {setAppError} = useAppErrors();
  const theme = useTheme();
  const layout = useWindowDimensions();
  const [index, setIndex] = useState(0);
  const [routes, setRoutes] = useState([
    {key: 'loan', title: 'Loan (0)'},
    {key: 'application', title: 'Application (0)'},
  ]);

  const getFilterArr = useCallback(
    (arr: ILoan[]) => {
      const ftc = arr.filter(f =>
        f.display_id.toLowerCase().includes(search.toLowerCase()),
      );
      return filter === FilteredLoanStatus.ALL_LOANS
        ? ftc
        : ftc.filter(f => f.status.toLowerCase() === filter.toLowerCase());
    },
    [filter, search],
  );

  const getAllLoans = useCallback(async () => {
    try {
      const {data} = await axiosInstance.get(
        `/v1/external-lending/sme/loans?${queryString.stringify(info || {})}`,
      );
      setFilteredData(data.loans);
    } catch (e: any) {
      setAppError(e.response.data);
    }
  }, [info, axiosInstance, setAppError]);

  useEffect(() => {
    getAllLoans();
  }, [getAllLoans]);

  const onFilter = (text: string) => {
    setFilter(text as FilteredLoanStatus);
    analytics.track(
      index === 0
        ? SEGMENT_EVENTS.CLICK_FILTER_LOANS_BY_STATUS
        : SEGMENT_EVENTS.CLICK_SEARCH_APPLICATION,
      {loan_status_selected: text},
    );
  };

  const loan = getFilterArr(
    filteredData.filter(item => APPLICATION_STATUS.includes(item.status)),
  );
  const application = getFilterArr(
    filteredData.filter(item => !APPLICATION_STATUS.includes(item.status)),
  );

  useEffect(() => {
    setRoutes([
      {key: 'loan', title: `Loan (${loan.length})`},
      {key: 'application', title: `Application (${application.length})`},
    ]);
  }, [loan.length, application.length]);

  const renderScene = SceneMap({
    loan: () => (
      <ScrollView style={styles.tabContent}>
        <LoansTable dataSource={loan} onPress={onPress} />
      </ScrollView>
    ),
    application: () => (
      <ScrollView style={styles.tabContent}>
        <ApplicationTable dataSource={application} onPress={onPress} />
      </ScrollView>
    ),
  });

  return (
    <View style={styles.container}>
      <View style={styles.searchContainer}>
        <TextInput
          mode="outlined"
          value={search}
          placeholder={'Search'}
          onChangeText={setSearch}
          style={styles.searchInput}
          onBlur={() => {
            analytics.track(
              index === 0
                ? SEGMENT_EVENTS.CLICK_SEARCH_LOAN
                : SEGMENT_EVENTS.CLICK_SEARCH_APPLICATION,
              {loan_id: search},
            );
          }}
        />
        <RNPickerSelect
          onValueChange={onFilter}
          style={pickerSelectStyles}
          items={FILTER_BY_STATUS}>
          <TouchableOpacity style={styles.filterButton}>
            <Text>Filter</Text>
          </TouchableOpacity>
        </RNPickerSelect>
      </View>
      <TabView
        navigationState={{index, routes}}
        renderScene={renderScene}
        onIndexChange={setIndex}
        initialLayout={{width: layout.width}}
        renderTabBar={props => (
          <TabBar
            {...props}
            indicatorStyle={{backgroundColor: theme.colors.primary}}
            style={{backgroundColor: theme.colors.background}}
            labelStyle={{
              color: '#000',
            }}
          />
        )}
      />
    </View>
  );
};

const styles = StyleSheet.create({
  container: {
    flex: 1,
    minHeight: 900,
  },
  searchContainer: {
    flexDirection: 'row',
    alignItems: 'center',
    margin: 10,
  },
  searchInput: {
    flex: 1,
    marginRight: 10,
  },
  filterButton: {
    backgroundColor: '#DCE3E8',
    borderRadius: 5,
    alignItems: 'center',
    padding: 20,
    width: '100%',
    justifyContent: 'center',
  },
  tabs: {
    backgroundColor: '#fff',
    flex: 1,
  },
  tabContent: {
    flex: 1,
    backgroundColor: 'white',
    padding: 20,
  },
});

const pickerSelectStyles = {
  inputIOS: {
    fontSize: 16,
    paddingVertical: 16,
    paddingHorizontal: 10,
    borderWidth: 1,
    borderColor: 'gray',
    borderRadius: 4,
    color: 'black',
    paddingRight: 30,
    backgroundColor: 'white',
  },
  inputAndroid: {
    fontSize: 16,
    paddingVertical: 12,
    paddingHorizontal: 10,
    borderWidth: 0.5,
    borderColor: 'gray',
    borderRadius: 8,
    color: 'black',
    paddingRight: 30,
    backgroundColor: 'white',
  },
};
