import React, { useCallback, useEffect, useState } from "react";
import { StyleSheet, View } from "react-native";
import { Formik } from "formik";
import * as Yup from "yup";
import {
  arrayLength,
  findKeysWithIsAttachment,
  phoneNumPatternForLogin,
  removeArrays,
  removeKeysWithIsAttachment,
} from "../helpers";
import { IBusiness } from "../typings/business";
import { showMessage } from "react-native-flash-message";
import analytics from "../utils/segment";
import {
  AttachementType,
  BUSINESS_TYPE,
  BusinessType,
  CBFS_INITIAL_DATA,
} from "../data";
import { SEGMENT_EVENTS } from "../data/segmentEvents";
import { PLATFORM } from "../data/constant";
import { BankProps } from "../typings/bank";
import { useLMS } from "../context/context";
import { Button, Text, TextInput } from "react-native-paper";
import { useAppErrors } from "../hooks";
import {
  FieldDropdownInput,
  FieldNumberInput,
  FieldTextInput,
  Gutter,
  UploadAttachment,
} from "../components";
import { KYBBusinessType } from "../typings/kyb";
import { Item } from "react-native-picker-select";

const attachmentSchema = Yup.object({
  values: Yup.array().min(1, () => {
    return "Required";
  }),
}).required();

const validationSchema = Yup.object().shape({
  fullName: Yup.string().required("Full Name is required"),
  email: Yup.string().email("valid email is required"),
  businessName: Yup.string().required("Business Name is required"),
  businessType: Yup.string().required(),
  businessAddress: Yup.string()
    .min(5, "Valid business address is required")
    .required("Business Address is required"),
  ntn: Yup.string()
    .min(7, "Valid NTN is required")
    .max(14, "Valid NTN is required")
    .required("NTN is required"),
  contactNumber: Yup.string()
    .matches(phoneNumPatternForLogin, "Contact Number is not valid")
    .min(10, "Contact Number should be 10 digits long.")
    .max(10, "Contact Number should be 10 digits long.")
    .required("Contact Number no is required"),
  partnerCnic: attachmentSchema.when("businessType", {
    is: "partnership",
    then: attachmentSchema.required("Partner CNIC is required"),
  }),
  partnershipDeed: attachmentSchema.when("businessType", {
    is: "partnership",
    then: attachmentSchema.required(),
  }),
  authorityLetter: attachmentSchema.when("businessType", {
    is: "partnership",
    then: attachmentSchema.required(),
  }),
  directorsResolution: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  memorandumArticlesAssociation: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  boardMemberCnic: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  pocCnic: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  beneficialOwnerCnic: attachmentSchema.when("businessType", {
    is: "company",
    then: attachmentSchema.required(),
  }),
  cnicFront: attachmentSchema.when("businessType", {
    is: "soleProprietorship" || "individual",
    then: attachmentSchema.required(),
  }),
  cnicBack: attachmentSchema.when("businessType", {
    is: "soleProprietorship" || "individual",
    then: attachmentSchema.required(),
  }),
  proprietorshipDeclaration: attachmentSchema.when("businessType", {
    is: "soleProprietorship",
    then: attachmentSchema.required(),
  }),
  ntnCertificate: attachmentSchema.when("businessType", {
    is: "soleProprietorship",
    then: attachmentSchema.required(),
  }),

  permissionLetter: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),
  signatoryPassports: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),
  formIIParticulars: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),
  principalOfficeAuthorization: attachmentSchema.when("businessType", {
    is: "branchOffice",
    then: attachmentSchema.required(),
  }),

  bankDetails: Yup.object().shape({
    bankCode: Yup.mixed().test("bankCode", (num, values) => {
      const {
        parent: { accountNumber, accountTitle, iban, bankCode },
      } = values;

      if (
        values.options.context &&
        values.options.context.isBankRequired &&
        !bankCode
      ) {
        return values.createError({
          path: values.path,
          message: "Bank is required",
        });
      }

      if (!!accountNumber || !!iban || !!accountTitle) {
        if (!bankCode || bankCode === "null") {
          return values.createError({
            path: values.path,
            message: "Bank is required",
          });
        }
      }
      return true;
    }),
    accountTitle: Yup.string().test("accountTitle", (num, values) => {
      const {
        parent: { accountNumber, accountTitle, iban, bankCode },
      } = values;

      if (
        values.options.context &&
        values.options.context.isBankRequired &&
        !accountTitle
      ) {
        return values.createError({
          path: values.path,
          message: "Account Title is required",
        });
      }

      if (!!accountNumber || !!iban || !!bankCode) {
        if (!accountTitle)
          return values.createError({
            path: values.path,
            message: "Account Title is required",
          });
      }
      return true;
    }),
    accountNumber: Yup.string().test("accountNumber", (num, values) => {
      const {
        parent: { accountNumber, accountTitle, iban, bankCode },
      } = values;

      if (
        values.options.context &&
        values.options.context.isBankRequired &&
        !accountNumber
      ) {
        return values.createError({
          path: values.path,
          message: "Account Number is required",
        });
      }

      if (!!accountNumber || !!accountTitle || !!iban || !!bankCode) {
        if (!accountNumber)
          return values.createError({
            path: values.path,
            message: "Account Number is required",
          });
      }
      return true;
    }),
    iban: Yup.string(),
  }),
});

export const RegisterFinance: React.ComponentType<{
  onSuccess?(business: IBusiness): void;
  isBankRequired?: boolean;
}> = ({ onSuccess, isBankRequired = false }) => {
  const [banks, setBanks] = useState<Item[]>([]);
  const [loading, setLoading] = useState(true);
  const { axiosInstance, setInfo, info, userDetail } = useLMS();
  //const theme = useTheme();
  const { setAppError } = useAppErrors();

  const onSubmit = async (value: any, action: any) => {
    try {
      action.setSubmitting(true);
      // Convert into desire format
      const tempValue: any = Object.keys(value).reduce((acc, key) => {
        if (value[key]?.isAttachment) {
          return {
            ...acc,
            [key]: value[key].values,
          };
        } else {
          return {
            ...acc,
            [key]: value[key],
          };
        }
      }, {});

      if (
        tempValue.bankDetails.accountTitle === "" &&
        tempValue.bankDetails.accountNumber === "" &&
        tempValue.bankDetails.iban === ""
      ) {
        delete tempValue.bankDetails;
      }

      const finalValue = {
        ...tempValue,
        contactNumber: "+92" + tempValue.contactNumber,
      };
      const documentLength = arrayLength(finalValue);

      const { data }: { data: IBusiness } = await axiosInstance.post(
        "/v1/external-lending/sme/users",
        finalValue,
      );
      showMessage({
        message: "Successfully created",
        type: "success",
      });

      analytics?.identify(data.kyb.id);
      onSuccess?.(data);
      setInfo?.({ email: data.kyb.email });
      action.resetForm({
        values: CBFS_INITIAL_DATA,
      });

      analytics?.track(SEGMENT_EVENTS.COMPLETE_KYC_REGISTRATION, {
        ...removeArrays(finalValue),
        ...PLATFORM,
        attached_documents_count: documentLength,
      });
    } catch (e: any) {
      setAppError(e?.response?.data, action);
    } finally {
      action.setSubmitting(false);
    }
  };

  const fetchBank = useCallback(async () => {
    try {
      const { data } = await axiosInstance.get(
        "/v1/external-lending/sme/banks",
      );
      setBanks(
        data.banks.map((m: BankProps) => {
          return { label: m.bank_name, value: m.bank_code };
        }),
      );
    } catch (e) {
      showMessage({
        message: "Unable to fetch bank list",
        type: "danger",
      });
    }
  }, [axiosInstance]);

  useEffect(() => {
    setLoading(false);
  }, [info, userDetail]);

  useEffect(() => {
    fetchBank();
  }, [fetchBank]);

  return (
    <View style={styles.container}>
      {!loading && (
        <Formik
          initialValues={{ ...CBFS_INITIAL_DATA, isBankRequired }}
          validationSchema={validationSchema}
          onSubmit={onSubmit}
        >
          {({
            values,
            handleSubmit,
            isSubmitting,
            setValues,
            setFieldValue,
            dirty,
            isValid,
          }) => (
            <View>
              <Text variant={"headlineSmall"} style={{ fontWeight: "500" }}>
                Register for Financing
              </Text>
              <Gutter spacing={0.1} />
              <Text variant={"bodyLarge"}>
                For helping us understand your business better, please provide
                the required detail
              </Text>
              <Gutter spacing={1.7} />
              <Text variant={"titleLarge"} style={{ fontWeight: "500" }}>
                Personal Details
              </Text>
              <Gutter spacing={0.6} />
              <FieldTextInput
                placeholder="Enter Full Name"
                name={"fullName"}
                title={"Full Name"}
                onBlur={() =>
                  analytics.track(SEGMENT_EVENTS.START_KYC_REGISTRATION, {
                    ...PLATFORM,
                  })
                }
              />
              <Gutter spacing={0.1} />
              <FieldNumberInput
                title="Contact Number"
                placeholder={"3001XXXXXX"}
                name={"contactNumber"}
                maxLength={10}
                left={<TextInput.Affix text="+92" />}
              />
              <Gutter spacing={0.1} />
              <FieldTextInput
                placeholder="Enter email address"
                name={"email"}
                style={{
                  width: "100%",
                }}
                title="Email"
              />

              <Gutter spacing={1.5} />
              <Text variant={"titleLarge"} style={{ fontWeight: "500" }}>
                Bank Details
              </Text>
              <Gutter spacing={0.6} />
              <FieldDropdownInput
                title={"Bank Name"}
                name={"bankDetails.bankCode"}
                items={banks}
                onValueChange={(value) => {
                  setFieldValue("bankDetails.bankCode", value || "");
                }}
              />
              <Gutter spacing={0.5} />
              <FieldTextInput
                name="bankDetails.accountTitle"
                placeholder={"Account Title"}
                title={"Account Title"}
              />
              <Gutter spacing={0.5} />
              <FieldNumberInput
                placeholder="Account Number"
                name={"bankDetails.accountNumber"}
                title="Account Number"
              />
              <Gutter spacing={0.5} />
              <FieldTextInput
                placeholder="IBAN"
                name={"bankDetails.iban"}
                title="IBAN"
              />
              <Gutter spacing={1.5} />
              <Text variant={"titleLarge"} style={{ fontWeight: "500" }}>
                Business Details
              </Text>
              <Gutter spacing={0.6} />
              <FieldTextInput
                placeholder="Business Name"
                name={"businessName"}
                title="Business Name"
              />
              <Gutter spacing={0.5} />
              <FieldDropdownInput
                title={"Business type"}
                name={"businessType"}
                items={BUSINESS_TYPE}
                onValueChange={(value) => {
                  setValues({
                    ...(removeKeysWithIsAttachment(values) as any),
                    businessType: value,
                    ...BusinessType[`${value}` as KYBBusinessType],
                  });
                }}
              />
              <FieldNumberInput
                placeholder={`Enter ${
                  values.businessType === "soleProprietorship" ||
                  values.businessType === "individual"
                    ? "CNIC"
                    : "NTN"
                } number`}
                name={"ntn"}
                title="NTN"
                maxLength={14}
              />
              <Gutter spacing={0.1} />
              <FieldTextInput
                placeholder={"Enter Business Address"}
                name={"businessAddress"}
                title={"Business Address"}
                multiline
                numberOfLines={2}
                style={{ height: 100 }}
              />
              <Gutter spacing={1.5} />
              <Text variant={"titleLarge"} style={{ fontWeight: "500" }}>
                Additional Details
              </Text>
              <Gutter spacing={0.5} />
              {findKeysWithIsAttachment(values).map((m) => {
                const attachment = values[m] as AttachementType;

                return (
                  <View style={styles.attachment} key={m}>
                    <UploadAttachment
                      name={`${m}.values`}
                      title={`${attachment.title}${
                        attachment.required ? "*" : ""
                      }`}
                    />
                  </View>
                );
              })}
              <Button
                style={styles.button}
                mode="contained"
                onPress={() => handleSubmit()}
                disabled={isSubmitting || !isValid || !dirty}
              >
                Submit
              </Button>
            </View>
          )}
        </Formik>
      )}
    </View>
  );
};

const styles = StyleSheet.create({
  container: {
    minHeight: 3000,
    display: "flex",
    flex: 1,
    padding: 16,
  },
  scrollViewContent: {
    padding: 16,
  },
  attachment: {
    marginBottom: 10,
  },
  button: { borderRadius: 4 },
});
