import React, { useCallback, useEffect, useMemo, useState } from "react";
import {
  Clipboard,
  ScrollView,
  StyleSheet,
  Text,
  TouchableOpacity,
  View,
} from "react-native";
import { Button, TextInput, useTheme } from "react-native-paper";
import { Formik } from "formik";
import moment from "moment";
import * as Yup from "yup";
import queryString from "query-string";
import { useLMS } from "../context/context";
import { useAppErrors } from "../hooks";
import { RepaymentType, TransactionType } from "../typings";
import analytics from "../utils/segment";
import { SEGMENT_EVENTS } from "../data/segmentEvents";
import { showMessage } from "react-native-flash-message";
import AntDesign from "react-native-vector-icons/AntDesign";
import {
  FieldDropdownInput,
  FieldTextInput,
  SingleAttachment,
} from "../components";
import { TRANSACTION_METHOD } from "../data";
import { FieldDatePicker } from "../components/molecules/FieldDatePicker";

const validationSchema = Yup.object().shape({
  amount: Yup.number()
    .required("Amount is required")
    .min(1, "Amount must be greater than zero")
    .test("maxPayment", (quantity, values) => {
      const { parent } = values;

      if (parent?.maxPayment < (quantity || 0)) {
        return values.createError({
          path: values.path,
          message: `Amount cannot be greater than ${parent?.maxPayment}`,
        });
      }
      return true;
    }),
  method: Yup.string()
    .required("Transaction Method is required")
    .typeError("Transaction Method is Required"),
});

type RepaymentModalProps = {
  onSuccess?(): void;
  loanId: string;
  visible: boolean;
  onDismiss(): void;
};

export const RepaymentModal: React.FC<RepaymentModalProps> = ({
  onDismiss,
  onSuccess,
  loanId,
}) => {
  const [loading, setLoading] = useState(true);
  const [detail, setDetail] = useState<any>();
  const [loanDetail, setLoanDetail] = useState<any>();
  const [isModalOpen, setIsModalOpen] = useState(false);
  const theme = useTheme();
  const { info, axiosInstance } = useLMS();
  const { setAppError } = useAppErrors();

  const getALoanDetail = useCallback(async () => {
    try {
      const { data: repaymentDetail } = await axiosInstance.get(
        `/v1/external-lending/sme/loans/repayment-details?${queryString.stringify(
          info || {},
        )}`,
      );
      const { data } = await axiosInstance.get(
        `/v1/external-lending/sme/loans/${loanId}`,
      );
      setDetail(repaymentDetail);
      setLoanDetail(data.loan);
    } catch (e: any) {
      setAppError(e.response.data);
    } finally {
      setLoading(false);
    }
  }, [info, axiosInstance, loanId, setAppError]);

  const initialValue: RepaymentType = useMemo(() => {
    return {
      user_id: loanDetail?.user_id ?? "",
      amount: "0",
      type: TransactionType.REPAYMENT,
      method: undefined,
      payment_date: moment.utc().format(),
      maxPayment: loanDetail?.pending?.principal || 0,
      transaction_image: "",
    };
  }, [loanDetail]);

  const onSubmit = useCallback(
    async (values: RepaymentType, actions: any) => {
      try {
        actions.setSubmitting(true);
        await axiosInstance.post(
          `/v1/external-lending/sme/loans/${loanId}/transaction`,
          values,
        );
        onSuccess?.();
        onDismiss?.();
        analytics.track(SEGMENT_EVENTS.SUCCESS_RECORD_REPAYMENT, {
          loan_id: loanId,
          repayment_amount: values.amount,
          transaction_method: values.method,
          payment_date: values.payment_date,
          file_attached: !!values.transaction_image,
        });
      } catch (e: any) {
        setAppError(e.response.data);
      } finally {
        actions.setSubmitting(false);
      }
    },
    [axiosInstance, loanId, onDismiss, onSuccess, setAppError],
  );

  useEffect(() => {
    if (loanId) {
      getALoanDetail();
    } else {
      showMessage({ message: "Invalid loan id", type: "danger" });
    }
  }, [getALoanDetail, loanId]);

  const Form = useCallback(() => {
    return (
      <View style={styles.formContainer}>
        {!loading && loanDetail && (
          <Formik
            initialValues={initialValue}
            onSubmit={onSubmit}
            validationSchema={validationSchema}
          >
            {({
              values,
              handleSubmit,
              isSubmitting,
              isValid,
              setFieldValue,
            }) => (
              <>
                <View style={styles.header}>
                  <Text style={styles.headerText}>Record Payment</Text>
                  <TouchableOpacity
                    onPress={() => {
                      setIsModalOpen(false);
                      onDismiss?.();
                    }}
                  >
                    <AntDesign name={"close"} size={25} />
                  </TouchableOpacity>
                </View>
                <View style={styles.spacing} />
                <FieldTextInput
                  title={"Amount"}
                  name={"amount"}
                  value={values.amount}
                  keyboardType={"numeric"}
                  right={<TextInput.Affix text="PKR" />}
                />
                <View style={styles.spacing} />
                <Text>Payment Date*</Text>
                <FieldDatePicker name={"payment_date"} />
                <View style={styles.spacing} />
                <FieldDropdownInput
                  title={"Transaction Method"}
                  name={"method"}
                  items={TRANSACTION_METHOD}
                  onValueChange={(value) => {
                    setFieldValue("method", value);
                  }}
                />
                <View style={styles.spacing} />
                <SingleAttachment
                  name="transaction_image"
                  title="Transaction Image (Optional)"
                />
                <View style={styles.spacingLarge} />
                <View style={styles.footer}>
                  {/*<Text></Text>*/}
                  <Button
                    style={{ width: "100%" }}
                    mode="contained"
                    onPress={() => handleSubmit()}
                    disabled={!isValid || isSubmitting}
                  >
                    Save
                  </Button>
                </View>
              </>
            )}
          </Formik>
        )}
      </View>
    );
  }, [initialValue, loading, loanDetail, onDismiss, onSubmit]);

  const Information = () => {
    return (
      <ScrollView style={styles.scrollView}>
        <View style={styles.header}>
          <Text style={styles.headerText}>Make Repayment</Text>
          <TouchableOpacity
            onPress={() => {
              setIsModalOpen(false);
              onDismiss?.();
            }}
          >
            <AntDesign name={"close"} size={25} />
          </TouchableOpacity>
        </View>
        <View style={styles.spacing} />
        {detail?.repayment_address && (
          <>
            <View style={[styles.repaymentAddress]}>
              <Text style={styles.placeholder}>Bank Name</Text>
              <Text style={styles.repaymentText}>
                {detail.repayment_address.bank_name}
              </Text>
              <View style={styles.spacingSmall} />
              <Text style={styles.placeholder}>Account Title</Text>
              <Text style={styles.repaymentText}>
                {detail.repayment_address.account_title}
              </Text>
              <View style={styles.spacingSmall} />
              <Text style={styles.placeholder}>Account Number</Text>
              <View style={styles.row}>
                <Text style={styles.repaymentText}>
                  {detail.repayment_address.account_number}
                </Text>
                <TouchableOpacity
                  onPress={() => {
                    Clipboard.setString(
                      detail.repayment_address.account_number,
                    );
                    showMessage({
                      message: "Account Number successfully copied",
                      type: "success",
                    });
                  }}
                >
                  <AntDesign name={"copy1"} style={styles.iconSpacing} />
                </TouchableOpacity>
              </View>
              <View style={styles.spacingSmall} />
              <Text style={styles.placeholder}>IBAN</Text>
              <View style={styles.row}>
                <Text style={styles.repaymentText}>
                  {detail.repayment_address.iban}
                </Text>
                <TouchableOpacity
                  onPress={() => {
                    Clipboard.setString(detail.repayment_address.iban);
                    showMessage({
                      message: "IBAN successfully copied",
                      type: "success",
                    });
                  }}
                >
                  <AntDesign name={"copy1"} style={styles.iconSpacing} />
                </TouchableOpacity>
              </View>
            </View>
            <View style={styles.spacing} />
          </>
        )}
        <View style={styles.instructions}>
          <View style={styles.instructionsHeader}>
            <AntDesign name={"infocirlce"} />
            <View style={styles.iconSpacing} />
            <Text
              style={[styles.repaymentText, { color: "rgb(124, 117, 126)" }]}
            >
              Repayment Instructions
            </Text>
          </View>
          <View style={styles.spacing} />
          <Text>
            A step by step guide on how to repay your financing amount.
          </Text>
          <View style={styles.spacing} />
          <Text>{detail?.instructions}</Text>
        </View>
        <View style={styles.spacing} />
        <Button
          mode="outlined"
          style={{ width: "100%" }}
          onPress={() => {
            analytics.track(SEGMENT_EVENTS.CLICK_RECORD_REPAYMENT, {
              loan_id: loanId,
            });
            setIsModalOpen(true);
          }}
        >
          Record Repayment
        </Button>
      </ScrollView>
    );
  };

  return (
    <>
      <View
        style={[
          styles.modalContainer,
          { backgroundColor: theme.colors.background },
        ]}
      >
        {isModalOpen && <Form />}
        {!isModalOpen && <Information />}
      </View>
    </>
  );
};

const styles = StyleSheet.create({
  formContainer: {
    width: "100%",
  },
  header: {
    flexDirection: "row",
    justifyContent: "space-between",
    width: "100%",
  },
  headerText: {
    fontWeight: "600",
    fontSize: 16,
  },
  spacing: {
    marginVertical: 8,
  },
  textInput: {
    width: "100%",
  },
  spacingLarge: {
    marginVertical: 16,
  },
  footer: {
    flexDirection: "row",
    justifyContent: "space-between",
    width: "100%",
  },
  scrollView: {
    width: "100%",
  },
  repaymentAddress: {
    padding: 10,
    borderWidth: 1,
    borderRadius: 5,
    borderColor: "#F7F7F7",
    backgroundColor: "#F7F7F7",
  },
  repaymentText: {
    fontWeight: "500",
  },
  spacingSmall: {
    marginVertical: 7,
  },
  placeholder: {
    color: "rgb(124, 117, 126)",
  },
  row: {
    flexDirection: "row",
    alignItems: "center",
  },
  iconSpacing: {
    marginLeft: 8,
  },
  instructions: {
    padding: 10,
    borderWidth: 1,
    borderRadius: 5,
    borderColor: "#F7F7F7",
  },
  instructionsHeader: {
    flexDirection: "row",
    justifyContent: "flex-start",
    alignItems: "center",
  },
  modalContainer: {
    alignItems: "flex-end",
    padding: 16,
    borderRadius: 10,
    minWidth: 300,
    maxWidth: 500,
    maxHeight: "90%",
    width: "100%",
  },
});
