import React, { SetStateAction } from "react";
import {
  ScrollView,
  StyleSheet,
  useWindowDimensions,
  View,
} from "react-native";
import { Button, Card, Chip, Text, useTheme } from "react-native-paper";
import { FinanceAppState, ILoan } from "../../typings/loan";
import analytics from "../../utils/segment";
import { SEGMENT_EVENTS } from "../../data/segmentEvents";
import { LOAN_COLOR } from "../../data/color";
import { pascalCase } from "../../helpers";
import { SceneMap, TabBar, TabView } from "react-native-tab-view";
import { Gutter } from "../../components";
import { LoanItemDetails } from "../components/LoanItemDetails";
import { LoanRepayment } from "../components/RepaymentsDetails";

type ApprovedLoanProps = {
  loanDetail: ILoan;
  onRecordClick?(): void;
  index: number;
  setValue: React.Dispatch<SetStateAction<number>>;
};

export const ApprovedLoan: React.FC<ApprovedLoanProps> = ({
  loanDetail,
  index = 0,
  onRecordClick,
  setValue,
}) => {
  const theme = useTheme();
  const layout = useWindowDimensions();
  const isVisible =
    loanDetail.status !== FinanceAppState.IN_PROGRESS &&
    loanDetail.status !== FinanceAppState.UNDER_REVIEW &&
    loanDetail.status !== FinanceAppState.APPROVED &&
    loanDetail.status !== FinanceAppState.READY_TO_DISBURSE;

  const routes = [
    { key: "Details", title: "Details" },
    { key: "Repayments", title: "Repayments" },
  ];

  const renderScene = SceneMap({
    Details: () => (
      <ScrollView style={styles.tabContent}>
        <LoanItemDetails {...loanDetail} />
      </ScrollView>
    ),
    Repayments: () => (
      <ScrollView style={styles.tabContent}>
        <LoanRepayment {...loanDetail} />
      </ScrollView>
    ),
  });

  return (
    <>
      <View style={styles.column}>
        <View style={styles.row}>
          <Text style={styles.title}>Loan {loanDetail.display_id}</Text>
          <Chip
            mode="outlined"
            style={styles.chip}
            textStyle={{ color: LOAN_COLOR[loanDetail.status] }}
          >
            {pascalCase(loanDetail.status.replaceAll("_", " "))}
          </Chip>
        </View>
        {isVisible && (
          <>
            <Card style={styles.innerCard} elevation={0}>
              <View style={styles.innerRow}>
                <View style={styles.row}>
                  <Text style={styles.headerText}>Payable Today:</Text>
                  <Text style={styles.headerText}>
                    PKR{" "}
                    {(
                      parseFloat(`${loanDetail.pending?.principal}` || "0") +
                      parseFloat(`${loanDetail.pending?.markup_fee}` || "0") +
                      parseFloat(`${loanDetail.pending?.late_fees}` || "0")
                    ).toFixed(2)}
                  </Text>
                </View>
                <Gutter spacing={0.5} />
                <View style={styles.row}>
                  <Text style={styles.captionText}>Total Principal:</Text>
                  <Text style={styles.captionText}>
                    PKR{" "}
                    {parseFloat(
                      `${loanDetail.required_financing}` || "0",
                    ).toFixed(2)}
                  </Text>
                </View>
                <Gutter spacing={0.5} />
                <View style={styles.row}>
                  <Text style={styles.captionText}>Markup Accrued:</Text>
                  <Text style={styles.captionText}>
                    PKR {parseFloat(`${loanDetail.markup}` || "0").toFixed(2)}
                  </Text>
                </View>
                <Gutter spacing={0.5} />

                {loanDetail.status === "disbursed" && (
                  <Button
                    mode="contained"
                    style={styles.button}
                    onPress={() => {
                      analytics.track(SEGMENT_EVENTS.CLICK_MAKE_REPAYMENT, {
                        loan_id: loanDetail._id,
                      });
                      onRecordClick?.();
                    }}
                  >
                    Make Payment
                  </Button>
                )}
              </View>
            </Card>
          </>
        )}
        <Gutter spacing={1} />
      </View>
      <TabView
        navigationState={{ index, routes }}
        renderScene={renderScene}
        onIndexChange={setValue}
        initialLayout={{ width: layout.width }}
        renderTabBar={(props: any) => (
          <TabBar
            {...props}
            indicatorStyle={{ backgroundColor: theme.colors.primary }}
            style={{ backgroundColor: theme.colors.background }}
            labelStyle={{
              color: "#000",
            }}
          />
        )}
      />
    </>
  );
};

const styles = StyleSheet.create({
  column: {
    flexDirection: "column",
  },
  row: {
    flexDirection: "row",
    justifyContent: "space-between",
    alignItems: "center",
  },
  title: {
    fontWeight: "600",
    fontSize: 18,
  },
  chip: {
    borderWidth: 1,
    borderRadius: 40,
  },
  innerCard: {
    width: "100%",
    marginTop: 16,
    padding: 16,
    borderWidth: 1,
  },
  innerRow: {
    flexDirection: "column",
    justifyContent: "space-between",
  },
  headerText: {
    fontWeight: "500",
    fontSize: 16,
  },
  tabContent: {
    flex: 1,
  },
  captionText: {
    fontSize: 14,
    color: "gray",
  },
  button: {
    borderRadius: 4,
    marginTop: 16,
  },
  tabsContainer: {
    width: "100%",
    marginTop: 16,
  },
  tabs: {
    backgroundColor: "white",
  },
});
