import * as React from 'react';
import {useCallback} from 'react';
import {StyleSheet, TouchableOpacity, View} from 'react-native';
import {Chip, Text} from 'react-native-paper';
import moment from 'moment';
import {FinanceAppState, ILoan, LoanListType} from '../../typings/loan';
import {pascalCase} from '../../helpers';
import {getDateFormat} from '../../helpers/date';
import {LOAN_COLOR} from '../../data/color';

const styles = StyleSheet.create({
  container: {
    flexDirection: 'column',
  },
  loanContainer: {
    marginBottom: 16,
    padding: 16,
    backgroundColor: 'white',
    borderRadius: 8,
    elevation: 3,
    borderWidth: 1,
    borderColor: '#E0E0E0',
  },
  flexRow: {
    flexDirection: 'row',
    justifyContent: 'space-between',
    marginBottom: 8,
  },
  chip: {
    borderWidth: 1,
    borderRadius: 40,
    backgroundColor: '#fff',
  },
});

type LoansTableProps = {
  dataSource: ILoan[];
  onPress?(type: LoanListType.loan, loan: ILoan): void;
};

export const LoansTable: React.FC<LoansTableProps> = ({
  dataSource = [],
  onPress,
}) => {
  const Render = useCallback(
    ({...row}: ILoan) => {
      return (
        <TouchableOpacity onPress={() => onPress?.(LoanListType.loan, row)}>
          <View key={row.display_id} style={styles.loanContainer}>
            <View style={styles.flexRow}>
              <Text>Loan:</Text>
              <Text>{row.display_id}</Text>
            </View>
            <View style={styles.flexRow}>
              <Text>Status:</Text>
              <Chip
                mode="outlined"
                style={[
                  styles.chip,
                  {
                    borderColor: LOAN_COLOR[row.status],
                  },
                ]}>
                <Text style={{color: LOAN_COLOR[row.status]}}>
                  {pascalCase(row.status.replaceAll('_', ' '))}
                </Text>
              </Chip>
            </View>
            <View style={styles.flexRow}>
              <Text>End Date:</Text>
              <Text>
                {row.status === FinanceAppState.DISBURSED
                  ? moment(row.due_date).format('DD/MM/YY')
                  : ''}
                <Text style={{color: 'red'}}>
                  {row.status === FinanceAppState.DISBURSED
                    ? getDateFormat(row?.due_date || '')
                    : ''}
                </Text>
              </Text>
            </View>
            <View style={styles.flexRow}>
              <Text>Principal (PKR):</Text>
              <Text>{(row.required_financing || 0).toLocaleString()}</Text>
            </View>
            <View style={styles.flexRow}>
              <Text>Markup (PKR):</Text>
              <Text>
                {row.status === FinanceAppState.DISBURSED
                  ? parseFloat(
                      `${row.pending?.markup_fee || 0}`,
                    ).toLocaleString()
                  : ''}
              </Text>
            </View>
            <View style={styles.flexRow}>
              <Text>Repaid (PKR):</Text>
              <Text>
                {row.status === FinanceAppState.DISBURSED
                  ? (
                      parseFloat(`${row.payments?.markup_fee || 0}`) +
                      parseFloat(`${row.payments?.principal_repaid || 0}`) +
                      parseFloat(`${row.payments?.late_fees || 0}`)
                    ).toLocaleString()
                  : ''}
              </Text>
            </View>
            <View style={styles.flexRow}>
              <Text>Late Fee (PKR):</Text>
              <Text>
                {row.status === FinanceAppState.DISBURSED
                  ? (
                      parseFloat(`${row.pending?.late_fees || 0}`) +
                      parseFloat(`${row.payments?.late_fees || 0}`)
                    ).toLocaleString()
                  : ''}
              </Text>
            </View>
            <View style={styles.flexRow}>
              <Text>Amount Due (PKR):</Text>
              <Text>
                {row.status === FinanceAppState.DISBURSED
                  ? (
                      parseFloat(`${row.pending?.markup_fee || 0}`) +
                      parseFloat(`${row.pending?.principal || 0}`) +
                      parseFloat(`${row.pending?.late_fees || 0}`)
                    ).toLocaleString()
                  : ''}
              </Text>
            </View>
          </View>
        </TouchableOpacity>
      );
    },
    [onPress],
  );

  return (
    <>
      {dataSource.map(row => (
        <Render key={row.display_id} {...row} />
      ))}
    </>
  );
};
